package org.java.evolutionary.sequence;

import org.biojava.bio.seq.DNATools;
import org.biojava.bio.symbol.FiniteAlphabet;
import org.biojava.bio.symbol.SymbolList;
import org.biojava.utils.regex.Matcher;
import org.biojava.utils.regex.Pattern;
import org.biojava.utils.regex.PatternFactory;

import java.util.regex.*;

//
//  FTNode.java
//  
//
//  Created by Jack Compton on 6/14/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

public class FTNode {
	public FTNode parent;
	public FTNode lchild;
	public FTNode rchild;
	public String s;
	public int position;
	public int proximity;
	public Pattern bp;
	public Pattern bp2;
	public String motif1;
	public String motif2;
	public int start;
	public int end;
	

	public FTNode() {
		parent = null;
		lchild = null;
		rchild = null;
	}

	public boolean value(SymbolList seq) {
		if (s.compareTo("AND") == 0) {
			boolean val = rchild.value(seq) && lchild.value(seq);
			return val;
		} else if (s.compareTo("Not") == 0) {
			boolean val = !(lchild.value(seq));
			return val;
		} else if (s.compareTo("OR") == 0) {
			boolean val = rchild.value(seq) || lchild.value(seq);
			return val;
		} else if (s.compareTo("true") == 0) {
			return true;
		} else if (s.length() > 19
				&& s.substring(0, 20).compareTo("( matchesAtPosition ") == 0) {
			return matches_motif(seq);
	    } 
		else if (s.length() > 15
				&& s.substring(0, 16).compareTo("( shiftPosition ") == 0) {
			return matches_shiftposition(seq);
	    }
		else if (s.length() > 14
				&& s.substring(0, 15).compareTo("( regionalDown ") == 0) {
			return matches_regional(seq);
	    }
		else if (s.length() > 12
				&& s.substring(0, 13).compareTo("( regionalUp ") == 0) {
			return matches_regional(seq);
	    }
		else if (s.length() > 11
				   && s.substring(0, 12).compareTo("( correlate ") == 0) {
			return correlate_motif(seq);
		} else if (s.length() > 9
				&& s.substring(0, 10).compareTo("( matches ") == 0) {
			return present_motif(seq);
		} else {
			return false;
		}
	}

	public void addChild(FTNode ftn) {
		if (lchild == null) {
			lchild = ftn;
		} else if (rchild == null) {
			rchild = ftn;
		}

	}

	public static FTNode load_tree(String feature, int[] fi,
			PatternFactory factory) {
		while (feature.charAt(fi[0]) == ' ' || feature.charAt(fi[0]) == ')') {
			fi[0]++;
			if (fi[0] >= feature.length() - 1) {
				return null;
			}
		}

		if (feature.substring(fi[0], fi[0] + 4).compareTo("true") == 0) {
			FTNode nd = new FTNode();
			nd.s = "true";
			fi[0] += 5;
			return nd;
		} else if (feature.substring(fi[0], fi[0] + 6).compareTo("( AND ") == 0) {
			FTNode nd = new FTNode();
			nd.s = "AND";
			fi[0] += 6;
			nd.addChild(load_tree(feature, fi, factory));
			nd.addChild(load_tree(feature, fi, factory));
			return nd;
		} else if (feature.substring(fi[0], fi[0] + 6).compareTo("( Not ") == 0) {
			FTNode nd = new FTNode();
			nd.s = "Not";
			fi[0] += 6;
			nd.addChild(load_tree(feature, fi, factory));
			return nd;
		} else if (feature.substring(fi[0], fi[0] + 5).compareTo("( OR ") == 0) {
			FTNode nd = new FTNode();
			nd.s = "OR";
			fi[0] += 5;
			nd.addChild(load_tree(feature, fi, factory));
			nd.addChild(load_tree(feature, fi, factory));
			return nd;
		} else if (feature.substring(fi[0]).length() >= 20 && feature.substring(fi[0], fi[0] + 20).compareTo("( matchesAtPosition ") == 0) {
			FTNode nd = new FTNode();
            java.util.regex.Pattern p = java.util.regex.Pattern
					.compile("@\\s+\\d+\\)");
            java.util.regex.Matcher m = p.matcher(feature);
			m.find(fi[0]);
			int e = m.end(); // end of motif expression
			nd.s = feature.substring(fi[0], e);
            fi[0] = e + 1;



			// extract motif and position
			//p = java.util.regex.Pattern.compile("node\\d+\\s+([^@\\)]+)\\)\\s+@\\s+(\\d+)");
            p = java.util.regex.Pattern.compile("motif\\d+\\s+([^@\\)]+)\\)\\s+@\\s+(\\d+)");
			m = p.matcher(nd.s);
			m.find();
			String currentMotif = nd.s.substring(m.start(1), m.end(1));
			String positionString = nd.s.substring(m.start(2), m.end(2));
			nd.position = new Integer(positionString.trim()).intValue();

			// set up biojava motif matcher
			try {
				nd.bp = factory.compile(currentMotif);
			} catch (Exception er) {
				er.printStackTrace();
			}
			return nd;
        } else if (feature.substring(fi[0]).length() >= 16 && feature.substring(fi[0], fi[0] + 16).compareTo("( shiftPosition ") == 0) {
			FTNode nd = new FTNode();
            java.util.regex.Pattern p = java.util.regex.Pattern
					.compile("@\\s+\\d+\\s+\\d+\\)");
            java.util.regex.Matcher m = p.matcher(feature);
			m.find(fi[0]);
			int e = m.end(); // end of motif expression
			nd.s = feature.substring(fi[0], e);
            fi[0] = e + 1;



			// extract motif and position
			//p = java.util.regex.Pattern.compile("node\\d+\\s+([^@\\)]+)\\)\\s+@\\s+(\\d+)");
            p = java.util.regex.Pattern.compile("motif\\d+\\s+([^@\\)]+)\\)\\s+@\\s+(\\d+)\\s+(\\d+)");
			m = p.matcher(nd.s);
			m.find();
			String currentMotif = nd.s.substring(m.start(1), m.end(1));
			String positionString = nd.s.substring(m.start(2), m.end(2));
			nd.position = new Integer(positionString.trim()).intValue();
			String proximityString = nd.s.substring(m.start(3), m.end(3));
			nd.proximity = new Integer(proximityString.trim()).intValue();
			
			// set up biojava motif matcher
			try {
				nd.bp = factory.compile(currentMotif);
			} catch (Exception er) {
				er.printStackTrace();
			}
			return nd;
        }
        else if (feature.substring(fi[0]).length() >= 15 && feature.substring(fi[0], fi[0] + 15).compareTo("( regionalDown ") == 0) {
			FTNode nd = new FTNode();
            java.util.regex.Pattern p = java.util.regex.Pattern
					.compile("@\\s+\\d+\\s+\\d+\\)");
            java.util.regex.Matcher m = p.matcher(feature);
			m.find(fi[0]);
			int e = m.end(); // end of motif expression
			nd.s = feature.substring(fi[0], e);
            fi[0] = e + 1;



			// extract motif and position
			//p = java.util.regex.Pattern.compile("node\\d+\\s+([^@\\)]+)\\)\\s+@\\s+(\\d+)");
            p = java.util.regex.Pattern.compile("motif\\d+\\s+([^@\\)]+)\\)\\s+@\\s+(\\d+)\\s+(\\d+)");
			m = p.matcher(nd.s);
			m.find();
			nd.motif1 = nd.s.substring(m.start(1), m.end(1));
			String startString = nd.s.substring(m.start(2), m.end(2));
			nd.start = new Integer(startString.trim()).intValue();
			String endString = nd.s.substring(m.start(3), m.end(3));
			nd.end = new Integer(endString.trim()).intValue();
			
			// set up biojava motif matcher
			try {
				nd.bp = factory.compile(nd.motif1);
			} catch (Exception er) {
				er.printStackTrace();
			}
			return nd;
        }
        else if (feature.substring(fi[0]).length() >= 13 && feature.substring(fi[0], fi[0] + 13).compareTo("( regionalUp ") == 0) {
			FTNode nd = new FTNode();
            java.util.regex.Pattern p = java.util.regex.Pattern
					.compile("@\\s+\\d+\\s+\\d+\\)");
            java.util.regex.Matcher m = p.matcher(feature);
			m.find(fi[0]);
			int e = m.end(); // end of motif expression
			nd.s = feature.substring(fi[0], e);
            fi[0] = e + 1;



			// extract motif and position
			//p = java.util.regex.Pattern.compile("node\\d+\\s+([^@\\)]+)\\)\\s+@\\s+(\\d+)");
            p = java.util.regex.Pattern.compile("motif\\d+\\s+([^@\\)]+)\\)\\s+@\\s+(\\d+)\\s+(\\d+)");
			m = p.matcher(nd.s);
			m.find();
			nd.motif1 = nd.s.substring(m.start(1), m.end(1));
			String startString = nd.s.substring(m.start(2), m.end(2));
			nd.start = new Integer(startString.trim()).intValue();
			String endString = nd.s.substring(m.start(3), m.end(3));
			nd.end = new Integer(endString.trim()).intValue();
			
			// set up biojava motif matcher
			try {
				nd.bp = factory.compile(nd.motif1);
			} catch (Exception er) {
				er.printStackTrace();
			}
			return nd;
        }
		
		else if (feature.substring(fi[0], fi[0] + 12).compareTo("( correlate ") == 0) {
			FTNode nd = new FTNode();
            java.util.regex.Pattern p = java.util.regex.Pattern
			.compile("@\\s+\\d+\\s+\\d+\\)");
            java.util.regex.Matcher m = p.matcher(feature);
			m.find(fi[0]);
			int e = m.end(); // end of correlate expression
			nd.s = feature.substring(fi[0], e);
			fi[0] = e + 1;
			
			// extract motif and position
			
			p = java.util.regex.Pattern
			.compile("motif\\d+\\s+([^@\\)]+)\\)\\s+\\(\\s+motif\\d+\\s+([^@\\)]+)\\)\\s+@\\s+(\\d+)\\s+(\\d+)");
			m = p.matcher(nd.s);
			m.find();
			nd.motif1 = nd.s.substring(m.start(1), m.end(1));
			nd.motif2 = nd.s.substring(m.start(2), m.end(2));
			String positionString = nd.s.substring(m.start(3), m.end(3));
			String proximityString = nd.s.substring(m.start(4), m.end(4));
			nd.position = new Integer(positionString.trim()).intValue();
			nd.proximity = new Integer(proximityString.trim()).intValue();
			
			// set up biojava motif matcher
			try {
				nd.bp = factory.compile(nd.motif1);
				nd.bp2 = factory.compile(nd.motif2);
			} catch (Exception er) {
				er.printStackTrace();
			}
			return nd;	
		} else if (feature.substring(fi[0], fi[0] + 10).compareTo("( matches ") == 0) {
			FTNode nd = new FTNode();
            java.util.regex.Pattern p = java.util.regex.Pattern
					.compile("motif\\d+\\s+([^@\\)]+)\\)");
            java.util.regex.Matcher m = p.matcher(feature);
			m.find(fi[0]);
			int e = m.end(); // end of motif expression
			nd.s = feature.substring(fi[0], e);
			fi[0] = e + 1;

			// extract motif and position

			p = java.util.regex.Pattern.compile("motif\\d+\\s+([^@\\)]+)\\)");
			m = p.matcher(nd.s);
			m.find();
			String currentMotif = nd.s.substring(m.start(1), m.end(1));

			// set up biojava motif matcher
			try {
				nd.bp = factory.compile(currentMotif);
			} catch (Exception er) {
				er.printStackTrace();
			}
			return nd;
		} else {
			return null;
		}
	}

	public String toString() {
		String ret = "";
		if (s.compareTo("AND") == 0) {
			return "( AND " + lchild.toString() + " " + rchild.toString() + ")";
		} else if (s.compareTo("OR") == 0) {
			return "( OR " + lchild.toString() + " " + rchild.toString() + ")";
		} else if (s.compareTo("Not") == 0) {
			return "( Not " + lchild.toString() + ")";
		} else if (s.compareTo("true") == 0) {
			return s;
		} else if (s.length() > 9
				&& s.substring(0, 10).compareTo("( present ") == 0) {
			return s;
		} else if (s.length() > 9
				&& s.substring(0, 10).compareTo("( matches ") == 0) {
			return s;
		} else {
			return null;
		}
	}

	public boolean isBoolOp(String s) {
		if (s.compareTo("AND") == 0 || s.compareTo("OR") == 0
				|| s.compareTo("Not") == 0) {
			return true;
		} else {
			return false;
		}
	}

	public boolean falseExp(FTNode n) {
		if (n.s.compareTo("Not") == 0 && n.lchild.s.compareTo("true") == 0)
			return true;
		return false;
	}

	public void copyNode(FTNode from, FTNode to) {
		to.s = from.s;
		if (to.s.length() > 9
				&& to.s.substring(0, 10).compareTo("( matches ") == 0) {
			to.position = from.position;
		}
		to.lchild = from.lchild;
		to.rchild = from.rchild;
	}

	public void cleanse() {
		if (s.compareTo("AND") == 0) {
			if (isBoolOp(lchild.s)) {
				lchild.cleanse();
			}
			if (isBoolOp(rchild.s)) {
				rchild.cleanse();
			}
			if (lchild.s.compareTo("true") == 0) {
				s = rchild.s;
				lchild = rchild.lchild;
				rchild = rchild.rchild;
			} else if (rchild.s.compareTo("true") == 0) {
				s = lchild.s;
				rchild = lchild.rchild;
				lchild = lchild.lchild;
			} else if (falseExp(lchild)) {
				copyNode(lchild, this);
			} else if (falseExp(rchild)) {
				copyNode(lchild, this);
			}
		} else if (s.compareTo("OR") == 0) {
			if (isBoolOp(lchild.s)) {
				lchild.cleanse();
			}
			if (isBoolOp(rchild.s)) {
				rchild.cleanse();
			}
			if (lchild.s.compareTo("true") == 0
					|| rchild.s.compareTo("true") == 0) {
				s = "true";
				lchild = null;
				rchild = null;
			} else if (falseExp(lchild)) {
				copyNode(rchild, this);
			} else if (falseExp(rchild)) {
				copyNode(lchild, this);
			}
		} else if (s.compareTo("Not") == 0) {
			if (lchild.s.compareTo("AND") == 0 || lchild.s.compareTo("OR") == 0) {
				lchild.cleanse();
			}
			if (lchild.s.compareTo("Not") == 0) {
				copyNode(lchild.lchild, this);
				if (isBoolOp(s))
					this.cleanse();
			}
		}
	}

	private boolean present_motif(SymbolList seq) {
		boolean matched = false;
		try {
			if (bp != null) {
				Matcher occurences = bp.matcher(seq);
				while (occurences.find()) {
					matched = true;
					break;

				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return matched;
	}
	
	private boolean matches_regional(SymbolList seq) {
		boolean matched = false;
		try {
			if (bp != null) {
				Matcher occurences = bp.matcher(seq);
				while (occurences.find()) {
					int positionCurrent = occurences.start();
					if(positionCurrent >= start && positionCurrent <= (end- this.motif1.length())){
						matched = true;
						break;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return matched;
	}

	private boolean matches_motif(SymbolList seq) {
		boolean matched = false;
		try {
			if (bp != null) {
				Matcher occurences = bp.matcher(seq);
				while (occurences.find()) {
					int current = occurences.start();
					if (current == position) {
						matched = true;
						break;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return matched;
	}
	
	
	private boolean matches_shiftposition(SymbolList seq) {
		boolean matched = false;
		try {
			if (bp != null) {
				Matcher occurences = bp.matcher(seq);
				while (occurences.find()) {
					int current = occurences.start();
					if (Math.abs(position-current) <= proximity) {
						matched = true;
						break;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return matched;
	}
	
	private boolean correlate_motif(SymbolList seq) {
		boolean matched = false;
		try {
			if (bp != null && bp2 != null) {
				Matcher occurences = bp.matcher(seq);
				while (occurences.find()) {
					int position1 = occurences.start();
					if (position1 == position) {
						occurences = bp2.matcher(seq);
						while (occurences.find()) {
							int position2 = occurences.start();
//							if ( Math.abs(position - (current + motif2.length())) < proximity || Math.abs(current - (position + motif1.length())) < proximity || 
//								(position > current && position < (current + motif2.length()) ) || (current > position && current < (position + motif1.length())) ) {
							if((position2 >= position1+ motif1.length()) && Math.abs((position1 + motif1.length())-position2) <= proximity
									||
									((position1 >= position2+ motif2.length()) &&Math.abs((position2 + motif2.length())-position1) <= proximity)){	
								matched = true;
								break;
							}
						}
						break;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return matched;
	}
	
}
