package org.java.evolutionary.sequence;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.FileWriter;
import java.io.BufferedWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.*;

import org.apache.commons.io.FileUtils;
import org.biojava.bio.seq.DNATools;
import org.biojava.bio.symbol.FiniteAlphabet;
import org.biojava.bio.symbol.SymbolList;
import org.biojava.utils.regex.Matcher;
import org.biojava.utils.regex.Pattern;
import org.biojava.utils.regex.PatternFactory;



public class SpliceFeatureGenerator {
 public boolean cleanOnly;
 //holds sequences in memory
 ArrayList sequencesList = new ArrayList();
 //holds the corresponding labels in memroy
 ArrayList labels = new ArrayList();
 public static PatternFactory factory;
 
 private int[] fi={0}; // feature index
 
 public void generateLibSVMFile(File gpFile) throws Exception{
  //this is file that has features as graphs/trees
   ArrayList featureTrees = new ArrayList();
   ArrayList featureStrings = new ArrayList();
   ArrayList svmLines = new ArrayList();
   try {
     BufferedReader r = new BufferedReader(new InputStreamReader(new FileInputStream(gpFile)));
     String line = r.readLine();
     FTNode ftn;
     int k = 0;
     while(line != null){
       fi[0] = 0;
       System.out.println("Loading: " + line);
       ftn = FTNode.load_tree(line, fi, factory);
       ftn.cleanse();
       if (featureStrings.contains(ftn.toString())==false) {
         featureTrees.add(ftn);
         featureStrings.add(ftn.toString());
         System.out.println("Loaded feature tree:" + k++);
       }
       line = r.readLine();
     }
     File cleanFeatures = new File("./SSfeaturesClean.txt");
     FileUtils.writeLines(cleanFeatures, featureTrees);
	 System.out.println("Clean features written...");
   } catch (Exception e) {
     e.printStackTrace();
   }
  
	 if (cleanOnly == true) {
		 System.out.println("Done.");
		 return;
	 }
	 
  Iterator keys = this.sequencesList.iterator();
  FileWriter f = new FileWriter("./gp.libsvm", true);
  BufferedWriter svmOut = new BufferedWriter(f);
  int j=0;
  //iterate all sequences positive and negative
  while(keys.hasNext()){
   String sequence = (String)keys.next();
   
   //get the label
   String label = (String)this.labels.get(j);
   String featureString = label;
   
   for(int i=0 ;i< featureTrees.size(); i++){
    
    SymbolList currentSequence = null;
    try{
     currentSequence = DNATools.createDNA(sequence);
    }
    catch(Exception e){
     e.printStackTrace();
    }
        
    //measure the match 
    if(((FTNode)featureTrees.get(i)).value(currentSequence)){
     featureString = featureString + " " + (i+1) +":" + 1;
    }
    else {
     featureString = featureString + " " + (i+1) +":" + 0;
    }
   }
   featureString = featureString +"\n";
   svmOut.write(featureString);
   svmOut.flush();
   System.out.println("Wrote to LibSVM file:" + j);
   j = j+1;
  }
  svmOut.close();
 }

 
 /**
  * This method reads the sequences from File with labels +1, -1 
  * and tries to put it in right buckets.
  * It also initializes factor for IUPAC parsing etc
  * @param fileName
  */
 public void setup(String fileName) {
  BufferedReader reader = null; 
  try {

   reader = new BufferedReader(new InputStreamReader(
     new FileInputStream(fileName)));

   String totalLine = reader.readLine();
   
   int i = 0;
   while (totalLine != null) {
    int startOfSpace = totalLine.indexOf(" ");
    String sequence = totalLine.substring(0, startOfSpace);
    String label = totalLine.substring(startOfSpace + 1, totalLine
      .length());
    sequencesList.add(sequence);
    labels.add(label);
    totalLine = reader.readLine();
    System.out.println("Completed:" + i++);
   }
  }
  catch(Exception e){
   e.printStackTrace();
  }
  //create IUPAC reader
  FiniteAlphabet iupac = DNATools.getDNA();
  factory = PatternFactory.makeFactory(iupac);
 }
 
 public static void main(String[] args){
  try{
	   //File f = new File("C:\\Research\\TCBB\\Results\\Runs\\HallOfFame-run5.txt");
	   File f = new File("C:\\Research\\Class-CS910\\ClassResults\\GPFeatureLog.txt");
     SpliceFeatureGenerator gen = new SpliceFeatureGenerator();
//	  if ("--clean-only".compareTo(args[0]) == 0) {
//		  gen.cleanOnly = true;
//	  } else {
		  gen.cleanOnly = false;  
//	  }
  // String sampleFile = "C:\\Research\\datasets\\SpliceData\\Samples\\Acceptors\\AcceptorSample0.txt";
		   String sampleFile = "C:\\Research\\datasets\\SpliceData\\C_Elegans\\Acceptor\\C_ElegansAcceptorSample7.fasta";

// String sampleFile = "C:\\Research\\datasets\\EColi-Promoter\\Promoter.csv";

   gen.setup(sampleFile);
   gen.generateLibSVMFile(f);
   
  }
  catch(Exception e){
   e.printStackTrace();
  }
 }

}
