package org.java.evolutionary.sequence;

import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.util.HashMap;
import java.util.Hashtable;

import ec.EvolutionState;
import ec.Group;
import ec.Individual;
import ec.Subpopulation;
/**
 * This class extends Subpopulation and decreases the size of it every time 
 * by 25% for example.
 * @author udaykamath
 *
 */
public class SubpopulationExt extends Subpopulation{
	static int generation =0;
	
	 public Group emptyClone()
    {
        try
            {
            Subpopulation p = (Subpopulation)clone();
            p.species = species;  // don't throw it away...maybe this is a bad idea...
            double factor = 0.1;
            if(generation == 1)
            	factor=0.45;
            System.out.println("Reducing by " + factor);
            int totalNumberOfIndividuals = (int)(individuals.length - factor*individuals.length);
            p.individuals = new Individual[totalNumberOfIndividuals];  // empty
            generation++;
            return p;   
            }
        catch (CloneNotSupportedException e) { throw new InternalError(); } // never happens
        }
	
	 public void populate(EvolutionState state, int thread)
     {
		  int len = individuals.length;           // original length of individual array
	        int start = 0;                                          // where to start filling new individuals in -- may get modified if we read some individuals in
	        
	        // should we load individuals from a file? -- duplicates are permitted
	        if (loadInds)
	            {
	            InputStream stream = state.parameters.getResource(file,null);
	            if (stream == null)
	                state.output.fatal("Could not load subpopulation from file", file);
	            
	            try { readSubpopulation(state, new LineNumberReader(new InputStreamReader(stream))); }
	            catch (IOException e) { state.output.fatal("An IOException occurred when trying to read from the file " + state.parameters.getString(file, null) + ".  The IOException was: \n" + e,
	                    file, null); }
	            
	            if (len < individuals.length)
	                {
	                state.output.message("Old subpopulation was of size " + len + ", expanding to size " + individuals.length);
	                return;
	                }
	            
	            if (len > individuals.length)   // the population was shrunk, there's more space yet
	                {
	                // What do we do with the remainder?
	                if (extraBehavior == TRUNCATE)
	                    {
	                    state.output.message("Old subpopulation was of size " + len + ", truncating to size " + individuals.length);
	                    return;  // we're done
	                    }
	                else if (extraBehavior == WRAP)
	                    {
	                    state.output.message("Only " + individuals.length + " individuals were read in.  Subpopulation will stay size " + len + 
	                        ", and the rest will be filled with copies of the read-in individuals.");
	                        
	                    Individual[] oldInds = individuals;
	                    individuals = new Individual[len];
	                    System.arraycopy(oldInds, 0, individuals, 0, oldInds.length);
	                    start = oldInds.length;
	                                
	                    int count = 0;
	                    for(int i = start; i < individuals.length; i++)
	                        {
	                        individuals[i] = (Individual)(individuals[count].clone());
	                        if (++count >= start) count = 0;
	                        }
	                    return;
	                    }
	                else // if (extraBehavior == FILL)
	                    {
	                    state.output.message("Only " + individuals.length + " individuals were read in.  Subpopulation will stay size " + len + 
	                        ", and the rest will be filled using randomly generated individuals.");
	                        
	                    Individual[] oldInds = individuals;
	                    individuals = new Individual[len];
	                    System.arraycopy(oldInds, 0, individuals, 0, oldInds.length);
	                    start = oldInds.length;
	                    // now go on to fill the rest below...
	                    }                       
	                }
	            }

	        // populating the remainder with random individuals
	        HashMap h = null;
	        if (numDuplicateRetries >= 1)
	            h = new HashMap((individuals.length - start) / 2);  // seems reasonable

	        for(int x=start;x<individuals.length;x++) 
	            {
	            for(int tries=0; 
	                tries <= /* Yes, I see that*/ numDuplicateRetries; 
	                tries++)
	                {
	                individuals[x] = species.newIndividual(state, thread);

	                if (numDuplicateRetries >= 1)
	                    {
	                    // check for duplicates
	                    Object o = h.get(individuals[x]);
	                    if (o == null) // found nothing, we're safe
	                        // hash it and go
	                        {
	                        h.put(individuals[x],individuals[x]);
	                        break;
	                        }
	                    }
	                }  // oh well, we tried to cut down the duplicates
	            }
	        generation ++;
     }
}
