package org.java.evolutionary.sequence.func;

import org.biojava.bio.symbol.SymbolList;
import org.biojava.utils.regex.Matcher;
import org.biojava.utils.regex.Pattern;
import org.biojava.utils.regex.PatternFactory;

import ec.EvolutionState;
import ec.Problem;
import org.java.evolutionary.sequence.SequenceClassificationProblem;
import org.java.evolutionary.sequence.MotifData;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;

public class RegionUpstreamMatch extends GPNode{
	
	public String toString() {
		return " regionalUp ";
	}

	public void checkConstraints(final EvolutionState state, final int tree,
			final GPIndividual typicalIndividual, final Parameter individualBase) {
		super.checkConstraints(state, tree, typicalIndividual, individualBase);
		if (children.length != 2)
			state.output.error("Incorrect number of children for node "
					+ toStringForError() + " at " + individualBase);
	}
	
	

	public void eval(final EvolutionState state, final int thread,
			final GPData input, final ADFStack stack,
			final GPIndividual individual, final Problem problem) {
		MotifData motifData = ((MotifData) (input));
		//child should give me motif
		children[0].eval(state, thread, input, stack, individual, problem);		
		//get the curernt motif
		char[] currentMotif = motifData.currentMotif;
		children[1].eval(state, thread, input, stack, individual, problem);
	    int startPosition = motifData.currentPosition;
	    int endPosition = motifData.currentPosition;

		//check for match using biojava
	    SequenceClassificationProblem spliceSetProblem =(SequenceClassificationProblem)problem;
		PatternFactory factory = spliceSetProblem.getFactory(thread);
		SymbolList sequence = spliceSetProblem.getCurrentSequence();
		boolean matches = matches(sequence,currentMotif,startPosition,endPosition,factory);
		motifData.matches = matches;
	}
	
	private boolean matches(SymbolList seq, char[] motif,  int start, int end,PatternFactory factory  ){
		boolean matched = false;
		try{
			//in the sequence find the match
			Pattern bp1 = factory.compile(new String(motif));
			Matcher occurences = bp1.matcher(seq);
			while (occurences.find()) {
				int positionCurrent = occurences.start();
				if(positionCurrent >= start && positionCurrent <= (end- motif.length)){
					matched = true;
					break;
				}
			}	    
		}catch(Exception e){
			e.printStackTrace();
		}
		return matched;
	}

}
